@@ -23,19 +23,27 @@
 #ifndef __NDPI_DEFINE_INCLUDE_FILE__
 #define __NDPI_DEFINE_INCLUDE_FILE__
 
+/*
+ * The #define below is used for apps that dynamically link with nDPI to make
+ * sure that datastructures and in sync across versions
+ */
+#define NDPI_API_VERSION 49328
+
 /*
   gcc -E -dM - < /dev/null |grep ENDIAN
 */
 
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
-#include <machine/endian.h>
+#if defined(__FreeBSD__) || defined(__NetBSD__)
+#include <sys/endian.h>
 #endif
 
 #ifdef __OpenBSD__
 #include <endian.h>
 #define __BYTE_ORDER BYTE_ORDER
 #if BYTE_ORDER == LITTLE_ENDIAN
+#ifndef __LITTLE_ENDIAN__
 #define __LITTLE_ENDIAN__
+#endif /* __LITTLE_ENDIAN__ */
 #else
 #define __BIG_ENDIAN__
 #endif/* BYTE_ORDER */
@@ -105,20 +113,11 @@
 #define NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_UDP (NDPI_SELECTION_BITMASK_PROTOCOL_IPV4_OR_IPV6 | NDPI_SELECTION_BITMASK_PROTOCOL_INT_UDP)
 #define NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP (NDPI_SELECTION_BITMASK_PROTOCOL_IPV4_OR_IPV6 | NDPI_SELECTION_BITMASK_PROTOCOL_INT_TCP_OR_UDP)
 
-
-#define NDPI_SELECTION_BITMASK_PROTOCOL_TCP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_TCP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
-#define NDPI_SELECTION_BITMASK_PROTOCOL_V6_TCP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_V6_TCP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
-#define NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
-
 /* does it make sense to talk about udp with payload ??? have you ever seen empty udp packets ? */
 #define NDPI_SELECTION_BITMASK_PROTOCOL_UDP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_UDP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
 #define NDPI_SELECTION_BITMASK_PROTOCOL_V6_UDP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_V6_UDP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
 #define NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_UDP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_UDP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
 
-#define NDPI_SELECTION_BITMASK_PROTOCOL_TCP_OR_UDP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_TCP_OR_UDP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
-#define NDPI_SELECTION_BITMASK_PROTOCOL_V6_TCP_OR_UDP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_V6_TCP_OR_UDP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
-#define NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP_WITH_PAYLOAD		(NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_OR_UDP | NDPI_SELECTION_BITMASK_PROTOCOL_HAS_PAYLOAD)
-
 #define NDPI_SELECTION_BITMASK_PROTOCOL_TCP_WITHOUT_RETRANSMISSION	(NDPI_SELECTION_BITMASK_PROTOCOL_TCP | NDPI_SELECTION_BITMASK_PROTOCOL_NO_TCP_RETRANSMISSION)
 #define NDPI_SELECTION_BITMASK_PROTOCOL_V6_TCP_WITHOUT_RETRANSMISSION	(NDPI_SELECTION_BITMASK_PROTOCOL_V6_TCP | NDPI_SELECTION_BITMASK_PROTOCOL_NO_TCP_RETRANSMISSION)
 #define NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_WITHOUT_RETRANSMISSION	(NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP | NDPI_SELECTION_BITMASK_PROTOCOL_NO_TCP_RETRANSMISSION)
@@ -154,7 +153,7 @@
 
 /* misc definitions */
 #define NDPI_DEFAULT_MAX_TCP_RETRANSMISSION_WINDOW_SIZE 0x10000
-
+#define NDPI_DEFAULT_MAX_NUM_PKTS_PER_FLOW_TO_DISSECT   32
 
 /* TODO: rebuild all memory areas to have a more aligned memory block here */
 
@@ -164,30 +163,17 @@
 #define MAX_PACKET_COUNTER                                   65000
 #define MAX_DEFAULT_PORTS                                        5
 
-#define NDPI_DIRECTCONNECT_CONNECTION_IP_TICK_TIMEOUT          600
-#define NDPI_IRC_CONNECTION_TIMEOUT                            120
-#define NDPI_GNUTELLA_CONNECTION_TIMEOUT                        60
-#define NDPI_BATTLEFIELD_CONNECTION_TIMEOUT                     60
-#define NDPI_THUNDER_CONNECTION_TIMEOUT                         30
-#define NDPI_RTSP_CONNECTION_TIMEOUT                             5
-#define NDPI_TVANTS_CONNECTION_TIMEOUT                           5
-#define NDPI_YAHOO_DETECT_HTTP_CONNECTIONS                       1
-#define NDPI_YAHOO_LAN_VIDEO_TIMEOUT                            30
-#define NDPI_ZATTOO_CONNECTION_TIMEOUT                         120
-#define NDPI_ZATTOO_FLASH_TIMEOUT                                5
-#define NDPI_JABBER_STUN_TIMEOUT                                30
-#define NDPI_JABBER_FT_TIMEOUT				         5
-#define NDPI_SOULSEEK_CONNECTION_IP_TICK_TIMEOUT               600
-
 #ifdef NDPI_ENABLE_DEBUG_MESSAGES
  #define NDPI_LOG(proto, m, log_level, args...)		                                 \
   {								                         \
     struct ndpi_detection_module_struct *mod = (struct ndpi_detection_module_struct*) m; \
-    if(mod != NULL && mod->ndpi_debug_printf != NULL)		                         \
-      (*(mod->ndpi_debug_printf))(proto, mod, log_level, __FILE__, __FUNCTION__, __LINE__, args); \
+    if(mod && mod->ndpi_log_level >= log_level) {	                                 \
+      if(mod != NULL && mod->ndpi_debug_printf != NULL)		                         \
+        (*(mod->ndpi_debug_printf))(proto, mod, log_level, __FILE__, __FUNCTION__, __LINE__, args); \
+    } \
   }
 
- /* We must define NDPI_CURRENT_PROTO before include ndpi_main.h !!! 
+ /* We must define NDPI_CURRENT_PROTO before include ndpi_main.h !!!
   *
   * #include "ndpi_protocol_ids.h"
   * #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_XXXX
@@ -196,7 +182,7 @@
   */
 
  #ifndef NDPI_CURRENT_PROTO
- #define NDPI_CURRENT_PROTO NDPI_PROTO_UNKNOWN
+ #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_UNKNOWN
  #endif
 
  #define NDPI_LOG_ERR(mod, args...)		                                 \
@@ -232,7 +218,11 @@
 # define NDPI_LOG_DBG2(...) {}
 # else
 # define NDPI_LOG(proto, mod, log_level, args...) { /* printf(args); */ }
-# define NDPI_LOG_ERR(mod, args...)  { printf(args); }
+# ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+#  define NDPI_LOG_ERR(mod, args...)  { printf(args); }
+# else
+#  define NDPI_LOG_ERR(mod, args...)  { /* printf(args); */ }
+# endif
 # define NDPI_LOG_INFO(mod, args...) { /* printf(args); */ }
 # define NDPI_LOG_DBG(mod,  args...) { /* printf(args); */ }
 # define NDPI_LOG_DBG2(mod, args...) { /* printf(args); */ }
@@ -240,6 +230,7 @@
 #endif							/* NDPI_ENABLE_DEBUG_MESSAGES */
 
 #define NDPI_EXCLUDE_PROTO(mod,flow) ndpi_exclude_protocol(mod, flow, NDPI_CURRENT_PROTO, __FILE__, __FUNCTION__, __LINE__)
+#define NDPI_EXCLUDE_PROTO_EXT(mod,flow,proto) ndpi_exclude_protocol(mod, flow, proto, __FILE__, __FUNCTION__, __LINE__)
 
 /**
  * macro for getting the string len of a static string
@@ -250,9 +241,10 @@
 
 /** macro to compare 2 IPv6 addresses with each other to identify the "smaller" IPv6 address  */
 #define NDPI_COMPARE_IPV6_ADDRESS_STRUCTS(x,y)  \
-  ((((u_int64_t *)(x))[0]) < (((u_int64_t *)(y))[0]) || ( (((u_int64_t *)(x))[0]) == (((u_int64_t *)(y))[0]) && (((u_int64_t *)(x))[1]) < (((u_int64_t *)(y))[1])) )
+  ((x.u6_addr.u6_addr64[0] < y.u6_addr.u6_addr64[0]) || ((x.u6_addr.u6_addr64[0] == y.u6_addr.u6_addr64[0]) && (x.u6_addr.u6_addr64[1] < y.u6_addr.u6_addr64[1])))
 
 #define NDPI_NUM_BITS              512
+#define NDPI_NUM_BITS_MASK         (512-1)
 
 #define NDPI_BITS /* 32 */ (sizeof(ndpi_ndpi_mask) * 8 /* number of bits in a byte */)        /* bits per mask */
 #define howmanybits(x, y)   (((x)+((y)-1))/(y))
@@ -274,12 +266,18 @@
 #define NDPI_BITMASK_SET_ALL(a)   NDPI_ONE(&a)
 #define NDPI_BITMASK_SET(a, b)    { memcpy(&a, &b, sizeof(NDPI_PROTOCOL_BITMASK)); }
 
+#define NDPI_SET_BIT(num, n)    num |= 1ULL << ( n )
+#define NDPI_CLR_BIT(num, n)    num &= ~(1ULL << ( n ))
+#define NDPI_CLR_BIT(num, n)    num &= ~(1ULL << ( n ))
+#define NDPI_ISSET_BIT(num, n)  (num & (1ULL << ( n )))
+#define NDPI_ZERO_BIT(num)      num = 0
+
 /* this is a very very tricky macro *g*,
  * the compiler will remove all shifts here if the protocol is static...
  */
-#define NDPI_ADD_PROTOCOL_TO_BITMASK(bmask,value)     NDPI_SET(&bmask,value)
-#define NDPI_DEL_PROTOCOL_FROM_BITMASK(bmask,value)   NDPI_CLR(&bmask,value)
-#define NDPI_COMPARE_PROTOCOL_TO_BITMASK(bmask,value) NDPI_ISSET(&bmask,value)
+#define NDPI_ADD_PROTOCOL_TO_BITMASK(bmask,value)     NDPI_SET(&bmask,   value & NDPI_NUM_BITS_MASK)
+#define NDPI_DEL_PROTOCOL_FROM_BITMASK(bmask,value)   NDPI_CLR(&bmask,   value & NDPI_NUM_BITS_MASK)
+#define NDPI_COMPARE_PROTOCOL_TO_BITMASK(bmask,value) NDPI_ISSET(&bmask, value & NDPI_NUM_BITS_MASK)
 
 #define NDPI_SAVE_AS_BITMASK(bmask,value)  { NDPI_ZERO(&bmask) ; NDPI_ADD_PROTOCOL_TO_BITMASK(bmask, value); }
 
@@ -302,17 +300,30 @@
 #define NDPI_SCTP_PROTOCOL_TYPE	   132
 #define NDPI_IPIP_PROTOCOL_TYPE    0x04
 #define NDPI_ICMPV6_PROTOCOL_TYPE  0x3a
+#define NDPI_PGM_PROTOCOL_TYPE     0x71
+#define NDPI_PIM_PROTOCOL_TYPE     0x67
+
+#define NDPI_ARRAY_LENGTH(array) (sizeof(array) / sizeof((array)[0]))
 
 /* the get_uXX will return raw network packet bytes !! */
-#define get_u_int8_t(X,O)  (*(u_int8_t *)(((u_int8_t *)X) + O))
-#define get_u_int16_t(X,O)  (*(u_int16_t *)(((u_int8_t *)X) + O))
-#define get_u_int32_t(X,O)  (*(u_int32_t *)(((u_int8_t *)X) + O))
-#define get_u_int64_t(X,O)  (*(u_int64_t *)(((u_int8_t *)X) + O))
+#define get_u_int8_t(X,O)   (*(u_int8_t  *)((&(((u_int8_t *)X)[O]))))
+#define get_u_int16_t(X,O)  (*(u_int16_t *)((&(((u_int8_t *)X)[O]))))
+#define get_u_int32_t(X,O)  (*(u_int32_t *)((&(((u_int8_t *)X)[O]))))
+#if defined(__arm__)
+#include <stdint.h>
+static inline uint64_t get_u_int64_t(const uint8_t* X, int O)
+{
+  uint64_t tmp;
+  memcpy(&tmp, X + O, sizeof(tmp));
+  return tmp;
+}
+#else
+#define get_u_int64_t(X,O)  (*(u_int64_t *)((&(((u_int8_t *)X)[O]))))
+#endif // __arm__
 
 /* new definitions to get little endian from network bytes */
 #define get_ul8(X,O) get_u_int8_t(X,O)
 
-
 #if defined(__LITTLE_ENDIAN__) || defined(_LITTLE_ENDIAN)
 #define get_l16(X,O)  get_u_int16_t(X,O)
 #define get_l32(X,O)  get_u_int32_t(X,O)
@@ -327,14 +338,196 @@
 /* define memory callback function */
 #define match_first_bytes(payload,st) (memcmp((payload),(st),(sizeof(st)-1))==0)
 
-#if defined(WIN32) && !defined(snprintf)
-#define snprintf _snprintf 
+#if defined(WIN32)
+#undef strtok_r
+#define strtok_r strtok_s
+
+#if BYTE_ORDER == LITTLE_ENDIAN
+#define le16toh(x) (x)
+#define le32toh(x) (x)
+#else
+#error "byte order not supported"
 #endif
 
-#define NDPI_MAX_DNS_REQUESTS                   16
+#endif /* WIN32 */
 
-#define NDPI_MAJOR                              2
-#define NDPI_MINOR                              4
+#define NDPI_MAX_DNS_REQUESTS                   16
+#define NDPI_MIN_NUM_STUN_DETECTION             8
+
+#define NDPI_MAJOR                              4
+#define NDPI_MINOR                              8
 #define NDPI_PATCH                              0
 
+/* IMPORTANT: order according to its severity */
+#define NDPI_CIPHER_SAFE                        0
+#define NDPI_CIPHER_WEAK                        1
+#define NDPI_CIPHER_INSECURE                    2
+
+#define NDPI_OPTIMAL_HLL_NUM_BUCKETS           16
+
+#define NDPI_MAX_NUM_TLS_APPL_BLOCKS            8
+
+//#define TLS_HANDLE_SIGNATURE_ALGORITMS 1
+
+#ifdef __APPLE__
+
+#include <libkern/OSByteOrder.h>
+
+#define htobe16(x) OSSwapHostToBigInt16(x)
+#define htole16(x) OSSwapHostToLittleInt16(x)
+#define be16toh(x) OSSwapBigToHostInt16(x)
+#define le16toh(x) OSSwapLittleToHostInt16(x)
+
+#define htobe32(x) OSSwapHostToBigInt32(x)
+#define htole32(x) OSSwapHostToLittleInt32(x)
+#define be32toh(x) OSSwapBigToHostInt32(x)
+#define le32toh(x) OSSwapLittleToHostInt32(x)
+
+#define htobe64(x) OSSwapHostToBigInt64(x)
+#define htole64(x) OSSwapHostToLittleInt64(x)
+#define be64toh(x) OSSwapBigToHostInt64(x)
+#define le64toh(x) OSSwapLittleToHostInt64(x)
+
+#endif /* __APPLE__ */
+
+
+#if defined(_WINDOWS_)
+
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+
+#ifndef htobe16
+#define htobe16(x) htons(x)
+#endif
+
+#ifndef htole16
+#define htole16(x) (x)
+#endif
+
+#ifndef be16toh
+#define be16toh(x) ntohs(x)
+#endif
+
+#ifndef le16toh
+#define le16toh(x) (x)
+#endif
+
+#ifndef htobe32
+#define htobe32(x) htonl(x)
+#endif
+
+#ifndef htole32
+#define htole32(x) (x)
+#endif
+
+#ifndef be32toh
+#define be32toh(x) ntohl(x)
+#endif
+
+#ifndef le32toh
+#define le32toh(x) (x)
+#endif
+
+#ifndef htobe64
+#define htobe64(x) ndpi_htonll(x)
+#endif
+
+#ifndef htole64
+#define htole64(x) (x)
+#endif
+
+#ifndef be64toh
+#define be64toh(x) ndpi_ntohll(x)
+#endif
+
+#ifndef le64toh
+#define le64toh(x) (x)
+#endif
+
+#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
+
+#ifndef htobe16
+#define htobe16(x) (x)
+#endif
+
+#ifndef htole16
+#define htole16(x) __builtin_bswap16(x)
+#endif
+
+#ifndef be16toh
+#define be16toh(x) (x)
+#endif
+
+#ifndef le16toh
+#define le16toh(x) __builtin_bswap16(x)
+#endif
+
+#ifndef htobe32
+#define htobe32(x) (x)
+#endif
+
+#ifndef htole32
+#define htole32(x) __builtin_bswap32(x)
+#endif
+
+#ifndef be32toh
+#define be32toh(x) (x)
+#endif
+
+#ifndef le32toh
+#define le32toh(x) __builtin_bswap32(x)
+#endif
+
+#ifndef htobe64
+#define htobe64(x) (x)
+#endif
+
+#ifndef htole64
+#define htole64(x) __builtin_bswap64(x)
+#endif
+
+#ifndef be64toh
+#define be64toh(x) (x)
+#endif
+
+#ifndef le64toh
+#define le64toh(x) __builtin_bswap64(x)
+#endif
+
+#else
+#error Unexpected __BYTE_ORDER__
+
+#endif /* __BYTE_ORDER__ */
+#endif /* __MINGW32__ */
+
+
+#ifndef ETH_ARP
+#define ETH_ARP                0x0806
+#endif
+
+#ifndef ETH_P_IP
+#define ETH_P_IP               0x0800 	/* IPv4 */
+#endif
+
+#ifndef ETH_P_IPV6
+#define ETH_P_IPV6	       0x86dd	/* IPv6 */
+#endif
+
+#ifndef ETH_P_VLAN
+#define ETH_P_VLAN             0x8100
+#endif
+
+#ifndef ETH_P_MPLS_UNI
+#define ETH_P_MPLS_UNI         0x8847
+#endif
+
+#ifndef ETH_P_MPLS_MULTI
+#define ETH_P_MPLS_MULTI       0x8848
+#endif
+
+#ifndef ETH_P_PPPoE
+#define ETH_P_PPPoE            0x8864
+#endif
+
+#define MAX_NBPF_CUSTOM_PROTO  8
+
 #endif /* __NDPI_DEFINE_INCLUDE_FILE__ */
